@@ -1,27 +1,27 @@
-use serde_json;
 use std::collections::HashMap;
 use std::fs::{self, File};
 use std::io::prelude::*;
 use std::str;
 
-use cargotest::cargo_process;
-use cargotest::support::git;
-use cargotest::support::paths;
-use cargotest::support::registry::{cksum, Package};
-use cargotest::support::{execs, project, ProjectBuilder};
-use hamcrest::assert_that;
+use serde::Serialize;
+
+use crate::support::cargo_process;
+use crate::support::git;
+use crate::support::paths;
+use crate::support::registry::{cksum, Package};
+use crate::support::{basic_manifest, project, ProjectBuilder};
 
 fn setup() {
     let root = paths::root();
     t!(fs::create_dir(&root.join(".cargo")));
     t!(t!(File::create(root.join(".cargo/config"))).write_all(
         br#"
-        [source.crates-io]
-        replace-with = 'my-awesome-local-registry'
+            [source.crates-io]
+            replace-with = 'my-awesome-local-registry'
 
-        [source.my-awesome-local-registry]
-        directory = 'index'
-    "#
+            [source.my-awesome-local-registry]
+            directory = 'index'
+        "#
     ));
 }
 
@@ -39,7 +39,7 @@ struct Checksum {
 impl VendorPackage {
     fn new(name: &str) -> VendorPackage {
         VendorPackage {
-            p: Some(project(&format!("index/{}", name))),
+            p: Some(project().at(&format!("index/{}", name))),
             cksum: Checksum {
                 package: Some(String::new()),
                 files: HashMap::new(),
@@ -60,6 +60,11 @@ impl VendorPackage {
         self
     }
 
+    fn no_manifest(mut self) -> Self {
+        self.p = self.p.map(|pb| pb.no_manifest());
+        self
+    }
+
     fn build(&mut self) {
         let p = self.p.take().unwrap();
         let json = serde_json::to_string(&self.cksum).unwrap();
@@ -72,7 +77,12 @@ impl VendorPackage {
 fn simple() {
     setup();
 
-    VendorPackage::new("foo")
+    VendorPackage::new("bar")
+        .file("Cargo.toml", &basic_manifest("bar", "0.1.0"))
+        .file("src/lib.rs", "pub fn bar() {}")
+        .build();
+
+    let p = project()
         .file(
             "Cargo.toml",
             r#"
@@ -80,46 +90,26 @@ fn simple() {
             name = "foo"
             version = "0.1.0"
             authors = []
-        "#,
-        )
-        .file("src/lib.rs", "pub fn foo() {}")
-        .build();
-
-    let p = project("bar")
-        .file(
-            "Cargo.toml",
-            r#"
-            [package]
-            name = "bar"
-            version = "0.1.0"
-            authors = []
 
             [dependencies]
-            foo = "0.1.0"
+            bar = "0.1.0"
         "#,
         )
         .file(
             "src/lib.rs",
-            r#"
-            extern crate foo;
-
-            pub fn bar() {
-                foo::foo();
-            }
-        "#,
+            "extern crate bar; pub fn foo() { bar::bar(); }",
         )
         .build();
 
-    assert_that(
-        p.cargo("build"),
-        execs().with_status(0).with_stderr(
+    p.cargo("build")
+        .with_stderr(
             "\
-[COMPILING] foo v0.1.0
-[COMPILING] bar v0.1.0 ([..]bar)
+[COMPILING] bar v0.1.0
+[COMPILING] foo v0.1.0 ([CWD])
 [FINISHED] [..]
 ",
-        ),
-    );
+        )
+        .run();
 }
 
 #[test]
@@ -127,15 +117,6 @@ fn simple_install() {
     setup();
 
     VendorPackage::new("foo")
-        .file(
-            "Cargo.toml",
-            r#"
-            [package]
-            name = "foo"
-            version = "0.1.0"
-            authors = []
-        "#,
-        )
         .file("src/lib.rs", "pub fn foo() {}")
         .build();
 
@@ -149,33 +130,26 @@ fn simple_install() {
             authors = []
 
             [dependencies]
-            foo = "0.1.0"
+            foo = "0.0.1"
         "#,
         )
         .file(
             "src/main.rs",
-            r#"
-            extern crate foo;
-
-            pub fn main() {
-                foo::foo();
-            }
-        "#,
+            "extern crate foo; pub fn main() { foo::foo(); }",
         )
         .build();
 
-    assert_that(
-        cargo_process().arg("install").arg("bar"),
-        execs().with_status(0).with_stderr(
+    cargo_process("install bar")
+        .with_stderr(
             "  Installing bar v0.1.0
-   Compiling foo v0.1.0
+   Compiling foo v0.0.1
    Compiling bar v0.1.0
-    Finished release [optimized] target(s) in [..] secs
+    Finished release [optimized] target(s) in [..]s
   Installing [..]bar[..]
 warning: be sure to add `[..]` to your PATH to be able to run the installed binaries
 ",
-        ),
-    );
+        )
+        .run();
 }
 
 #[test]
@@ -183,15 +157,6 @@ fn simple_install_fail() {
     setup();
 
     VendorPackage::new("foo")
-        .file(
-            "Cargo.toml",
-            r#"
-            [package]
-            name = "foo"
-            version = "0.1.0"
-            authors = []
-        "#,
-        )
         .file("src/lib.rs", "pub fn foo() {}")
         .build();
 
@@ -211,29 +176,24 @@ fn simple_install_fail() {
         )
         .file(
             "src/main.rs",
-            r#"
-            extern crate foo;
-
-            pub fn main() {
-                foo::foo();
-            }
-        "#,
+            "extern crate foo; pub fn main() { foo::foo(); }",
         )
         .build();
 
-    assert_that(
-        cargo_process().arg("install").arg("bar"),
-        execs().with_status(101).with_stderr(
+    cargo_process("install bar")
+        .with_status(101)
+        .with_stderr(
             "  Installing bar v0.1.0
 error: failed to compile `bar v0.1.0`, intermediate artifacts can be found at `[..]`
 
 Caused by:
   no matching package named `baz` found
 location searched: registry `https://github.com/rust-lang/crates.io-index`
+perhaps you meant: bar or foo
 required by package `bar v0.1.0`
 ",
-        ),
-    );
+        )
+        .run();
 }
 
 #[test]
@@ -241,15 +201,6 @@ fn install_without_feature_dep() {
     setup();
 
     VendorPackage::new("foo")
-        .file(
-            "Cargo.toml",
-            r#"
-            [package]
-            name = "foo"
-            version = "0.1.0"
-            authors = []
-        "#,
-        )
         .file("src/lib.rs", "pub fn foo() {}")
         .build();
 
@@ -263,7 +214,7 @@ fn install_without_feature_dep() {
             authors = []
 
             [dependencies]
-            foo = "0.1.0"
+            foo = "0.0.1"
             baz = { version = "9.8.7", optional = true }
 
             [features]
@@ -272,78 +223,77 @@ fn install_without_feature_dep() {
         )
         .file(
             "src/main.rs",
-            r#"
-            extern crate foo;
-
-            pub fn main() {
-                foo::foo();
-            }
-        "#,
+            "extern crate foo; pub fn main() { foo::foo(); }",
         )
         .build();
 
-    assert_that(
-        cargo_process().arg("install").arg("bar"),
-        execs().with_status(0).with_stderr(
+    cargo_process("install bar")
+        .with_stderr(
             "  Installing bar v0.1.0
-   Compiling foo v0.1.0
+   Compiling foo v0.0.1
    Compiling bar v0.1.0
-    Finished release [optimized] target(s) in [..] secs
+    Finished release [optimized] target(s) in [..]s
   Installing [..]bar[..]
 warning: be sure to add `[..]` to your PATH to be able to run the installed binaries
 ",
-        ),
-    );
+        )
+        .run();
 }
 
 #[test]
 fn not_there() {
     setup();
 
-    let _ = project("index").build();
+    let _ = project().at("index").build();
 
-    let p = project("bar")
+    let p = project()
         .file(
             "Cargo.toml",
             r#"
             [package]
-            name = "bar"
+            name = "foo"
             version = "0.1.0"
             authors = []
 
             [dependencies]
-            foo = "0.1.0"
+            bar = "0.1.0"
         "#,
         )
         .file(
             "src/lib.rs",
-            r#"
-            extern crate foo;
-
-            pub fn bar() {
-                foo::foo();
-            }
-        "#,
+            "extern crate bar; pub fn foo() { bar::bar(); }",
         )
         .build();
 
-    assert_that(
-        p.cargo("build"),
-        execs().with_status(101).with_stderr(
+    p.cargo("build")
+        .with_status(101)
+        .with_stderr(
             "\
-error: no matching package named `foo` found
+error: no matching package named `bar` found
 location searched: [..]
-required by package `bar v0.1.0 ([..])`
+required by package `foo v0.1.0 ([..])`
 ",
-        ),
-    );
+        )
+        .run();
 }
 
 #[test]
 fn multiple() {
     setup();
 
-    VendorPackage::new("foo-0.1.0")
+    VendorPackage::new("bar-0.1.0")
+        .file("Cargo.toml", &basic_manifest("bar", "0.1.0"))
+        .file("src/lib.rs", "pub fn bar() {}")
+        .file(".cargo-checksum", "")
+        .build();
+
+    VendorPackage::new("bar-0.2.0")
+        .file("Cargo.toml", &basic_manifest("bar", "0.2.0"))
+        .file("src/lib.rs", "pub fn bar() {}")
+        .file(".cargo-checksum", "")
+        .build();
+
+    let p = project()
         .file(
             "Cargo.toml",
             r#"
@@ -351,159 +301,31 @@ fn multiple() {
             name = "foo"
             version = "0.1.0"
             authors = []
-        "#,
-        )
-        .file("src/lib.rs", "pub fn foo() {}")
-        .file(".cargo-checksum", "")
-        .build();
-
-    VendorPackage::new("foo-0.2.0")
-        .file(
-            "Cargo.toml",
-            r#"
-            [package]
-            name = "foo"
-            version = "0.2.0"
-            authors = []
-        "#,
-        )
-        .file("src/lib.rs", "pub fn foo() {}")
-        .file(".cargo-checksum", "")
-        .build();
-
-    let p = project("bar")
-        .file(
-            "Cargo.toml",
-            r#"
-            [package]
-            name = "bar"
-            version = "0.1.0"
-            authors = []
 
             [dependencies]
-            foo = "0.1.0"
+            bar = "0.1.0"
         "#,
         )
         .file(
             "src/lib.rs",
-            r#"
-            extern crate foo;
-
-            pub fn bar() {
-                foo::foo();
-            }
-        "#,
+            "extern crate bar; pub fn foo() { bar::bar(); }",
         )
         .build();
 
-    assert_that(
-        p.cargo("build"),
-        execs().with_status(0).with_stderr(
+    p.cargo("build")
+        .with_stderr(
             "\
-[COMPILING] foo v0.1.0
-[COMPILING] bar v0.1.0 ([..]bar)
+[COMPILING] bar v0.1.0
+[COMPILING] foo v0.1.0 ([CWD])
 [FINISHED] [..]
 ",
-        ),
-    );
+        )
+        .run();
 }
 
 #[test]
 fn crates_io_then_directory() {
-    let p = project("bar")
-        .file(
-            "Cargo.toml",
-            r#"
-            [package]
-            name = "bar"
-            version = "0.1.0"
-            authors = []
-
-            [dependencies]
-            foo = "0.1.0"
-        "#,
-        )
-        .file(
-            "src/lib.rs",
-            r#"
-            extern crate foo;
-
-            pub fn bar() {
-                foo::foo();
-            }
-        "#,
-        )
-        .build();
-
-    let cksum = Package::new("foo", "0.1.0")
-        .file("src/lib.rs", "pub fn foo() -> u32 { 0 }")
-        .publish();
-
-    assert_that(
-        p.cargo("build"),
-        execs().with_status(0).with_stderr(
-            "\
-[UPDATING] registry `[..]`
-[DOWNLOADING] foo v0.1.0 ([..])
-[COMPILING] foo v0.1.0
-[COMPILING] bar v0.1.0 ([..]bar)
-[FINISHED] [..]
-",
-        ),
-    );
-
-    setup();
-
-    let mut v = VendorPackage::new("foo");
-    v.file(
-        "Cargo.toml",
-        r#"
-        [package]
-        name = "foo"
-        version = "0.1.0"
-        authors = []
-    "#,
-    );
-    v.file("src/lib.rs", "pub fn foo() -> u32 { 1 }");
-    v.cksum.package = Some(cksum);
-    v.build();
-
-    assert_that(
-        p.cargo("build"),
-        execs().with_status(0).with_stderr(
-            "\
-[COMPILING] foo v0.1.0
-[COMPILING] bar v0.1.0 ([..]bar)
-[FINISHED] [..]
-",
-        ),
-    );
-}
-
-#[test]
-fn crates_io_then_bad_checksum() {
-    let p = project("bar")
-        .file(
-            "Cargo.toml",
-            r#"
-            [package]
-            name = "bar"
-            version = "0.1.0"
-            authors = []
-
-            [dependencies]
-            foo = "0.1.0"
-        "#,
-        )
-        .file("src/lib.rs", "")
-        .build();
-
-    Package::new("foo", "0.1.0").publish();
-
-    assert_that(p.cargo("build"), execs().with_status(0));
-    setup();
-
-    VendorPackage::new("foo")
+    let p = project()
         .file(
             "Cargo.toml",
             r#"
@@ -511,35 +333,113 @@ fn crates_io_then_bad_checksum() {
             name = "foo"
             version = "0.1.0"
             authors = []
+
+            [dependencies]
+            bar = "0.1.0"
+        "#,
+        )
+        .file(
+            "src/lib.rs",
+            "extern crate bar; pub fn foo() { bar::bar(); }",
+        )
+        .build();
+
+    let cksum = Package::new("bar", "0.1.0")
+        .file("src/lib.rs", "pub fn bar() -> u32 { 0 }")
+        .publish();
+
+    p.cargo("build")
+        .with_stderr(
+            "\
+[UPDATING] `[..]` index
+[DOWNLOADING] crates ...
+[DOWNLOADED] bar v0.1.0 ([..])
+[COMPILING] bar v0.1.0
+[COMPILING] foo v0.1.0 ([CWD])
+[FINISHED] [..]
+",
+        )
+        .run();
+
+    setup();
+
+    let mut v = VendorPackage::new("bar");
+    v.file("Cargo.toml", &basic_manifest("bar", "0.1.0"));
+    v.file("src/lib.rs", "pub fn bar() -> u32 { 1 }");
+    v.cksum.package = Some(cksum);
+    v.build();
+
+    p.cargo("build")
+        .with_stderr(
+            "\
+[COMPILING] bar v0.1.0
+[COMPILING] foo v0.1.0 ([CWD])
+[FINISHED] [..]
+",
+        )
+        .run();
+}
+
+#[test]
+fn crates_io_then_bad_checksum() {
+    let p = project()
+        .file(
+            "Cargo.toml",
+            r#"
+            [package]
+            name = "foo"
+            version = "0.1.0"
+            authors = []
+
+            [dependencies]
+            bar = "0.1.0"
         "#,
         )
         .file("src/lib.rs", "")
         .build();
 
-    assert_that(
-        p.cargo("build"),
-        execs().with_status(101).with_stderr(
+    Package::new("bar", "0.1.0").publish();
+
+    p.cargo("build").run();
+    setup();
+
+    VendorPackage::new("bar")
+        .file("Cargo.toml", &basic_manifest("bar", "0.1.0"))
+        .file("src/lib.rs", "")
+        .build();
+
+    p.cargo("build")
+        .with_status(101)
+        .with_stderr(
             "\
-error: checksum for `foo v0.1.0` changed between lock files
+error: checksum for `bar v0.1.0` changed between lock files
 
 this could be indicative of a few possible errors:
 
     * the lock file is corrupt
-    * a replacement source in use (e.g. a mirror) returned a different checksum
+    * a replacement source in use (e.g., a mirror) returned a different checksum
     * the source itself may be corrupt in one way or another
 
-unable to verify that `foo v0.1.0` is the same as when the lockfile was generated
+unable to verify that `bar v0.1.0` is the same as when the lockfile was generated
 
 ",
-        ),
-    );
+        )
+        .run();
 }
 
 #[test]
 fn bad_file_checksum() {
     setup();
 
-    VendorPackage::new("foo")
+    VendorPackage::new("bar")
+        .file("Cargo.toml", &basic_manifest("bar", "0.1.0"))
+        .file("src/lib.rs", "")
+        .build();
+
+    let mut f = t!(File::create(paths::root().join("index/bar/src/lib.rs")));
+    t!(f.write_all(b"fn bar() -> u32 { 0 }"));
+
+    let p = project()
         .file(
             "Cargo.toml",
             r#"
@@ -547,33 +447,17 @@ fn bad_file_checksum() {
             name = "foo"
             version = "0.1.0"
             authors = []
-        "#,
-        )
-        .file("src/lib.rs", "")
-        .build();
-
-    let mut f = t!(File::create(paths::root().join("index/foo/src/lib.rs")));
-    t!(f.write_all(b"fn foo() -> u32 { 0 }"));
-
-    let p = project("bar")
-        .file(
-            "Cargo.toml",
-            r#"
-            [package]
-            name = "bar"
-            version = "0.1.0"
-            authors = []
 
             [dependencies]
-            foo = "0.1.0"
+            bar = "0.1.0"
         "#,
         )
         .file("src/lib.rs", "")
         .build();
 
-    assert_that(
-        p.cargo("build"),
-        execs().with_status(101).with_stderr(
+    p.cargo("build")
+        .with_status(101)
+        .with_stderr(
             "\
 error: the listed checksum of `[..]lib.rs` has changed:
 expected: [..]
@@ -583,15 +467,24 @@ directory sources are not intended to be edited, if modifications are \
 required then it is recommended that [replace] is used with a forked copy of \
 the source
 ",
-        ),
-    );
+        )
+        .run();
 }
 
 #[test]
 fn only_dot_files_ok() {
     setup();
 
+    VendorPackage::new("bar")
+        .file("Cargo.toml", &basic_manifest("bar", "0.1.0"))
+        .file("src/lib.rs", "")
+        .build();
     VendorPackage::new("foo")
+        .no_manifest()
+        .file(".bar", "")
+        .build();
+
+    let p = project()
         .file(
             "Cargo.toml",
             r#"
@@ -599,36 +492,32 @@ fn only_dot_files_ok() {
             name = "foo"
             version = "0.1.0"
             authors = []
-        "#,
-        )
-        .file("src/lib.rs", "")
-        .build();
-    VendorPackage::new("bar").file(".foo", "").build();
-
-    let p = project("bar")
-        .file(
-            "Cargo.toml",
-            r#"
-            [package]
-            name = "bar"
-            version = "0.1.0"
-            authors = []
 
             [dependencies]
-            foo = "0.1.0"
+            bar = "0.1.0"
         "#,
         )
         .file("src/lib.rs", "")
         .build();
 
-    assert_that(p.cargo("build"), execs().with_status(0));
+    p.cargo("build").run();
 }
 
 #[test]
 fn random_files_ok() {
     setup();
 
+    VendorPackage::new("bar")
+        .file("Cargo.toml", &basic_manifest("bar", "0.1.0"))
+        .file("src/lib.rs", "")
+        .build();
     VendorPackage::new("foo")
+        .no_manifest()
+        .file("bar", "")
+        .file("../test", "")
+        .build();
+
+    let p = project()
         .file(
             "Cargo.toml",
             r#"
@@ -636,63 +525,32 @@ fn random_files_ok() {
             name = "foo"
             version = "0.1.0"
             authors = []
-        "#,
-        )
-        .file("src/lib.rs", "")
-        .build();
-    VendorPackage::new("bar")
-        .file("foo", "")
-        .file("../test", "")
-        .build();
-
-    let p = project("bar")
-        .file(
-            "Cargo.toml",
-            r#"
-            [package]
-            name = "bar"
-            version = "0.1.0"
-            authors = []
 
             [dependencies]
-            foo = "0.1.0"
+            bar = "0.1.0"
         "#,
         )
         .file("src/lib.rs", "")
         .build();
 
-    assert_that(p.cargo("build"), execs().with_status(0));
+    p.cargo("build").run();
 }
 
 #[test]
 fn git_lock_file_doesnt_change() {
     let git = git::new("git", |p| {
-        p.file(
-            "Cargo.toml",
-            r#"
-            [project]
-            name = "git"
-            version = "0.5.0"
-            authors = []
-        "#,
-        ).file("src/lib.rs", "")
-    }).unwrap();
+        p.file("Cargo.toml", &basic_manifest("git", "0.5.0"))
+            .file("src/lib.rs", "")
+    })
+    .unwrap();
 
     VendorPackage::new("git")
-        .file(
-            "Cargo.toml",
-            r#"
-            [package]
-            name = "git"
-            version = "0.5.0"
-            authors = []
-        "#,
-        )
+        .file("Cargo.toml", &basic_manifest("git", "0.5.0"))
         .file("src/lib.rs", "")
         .disable_checksum()
         .build();
 
-    let p = project("bar")
+    let p = project()
         .file(
             "Cargo.toml",
             &format!(
@@ -711,15 +569,15 @@ fn git_lock_file_doesnt_change() {
         .file("src/lib.rs", "")
         .build();
 
-    assert_that(p.cargo("build"), execs().with_status(0));
+    p.cargo("build").run();
 
     let mut lock1 = String::new();
     t!(t!(File::open(p.root().join("Cargo.lock"))).read_to_string(&mut lock1));
 
     let root = paths::root();
     t!(fs::create_dir(&root.join(".cargo")));
-    t!(
-        t!(File::create(root.join(".cargo/config"))).write_all(&format!(
+    t!(t!(File::create(root.join(".cargo/config"))).write_all(
+        format!(
             r#"
         [source.my-git-repo]
         git = '{}'
@@ -729,19 +587,19 @@ fn git_lock_file_doesnt_change() {
         directory = 'index'
     "#,
             git.url()
-        ).as_bytes())
-    );
+        )
+        .as_bytes()
+    ));
 
-    assert_that(
-        p.cargo("build"),
-        execs().with_status(0).with_stderr(
+    p.cargo("build")
+        .with_stderr(
             "\
 [COMPILING] [..]
 [COMPILING] [..]
 [FINISHED] [..]
 ",
-        ),
-    );
+        )
+        .run();
 
     let mut lock2 = String::new();
     t!(t!(File::open(p.root().join("Cargo.lock"))).read_to_string(&mut lock2));
@@ -751,20 +609,12 @@ fn git_lock_file_doesnt_change() {
 #[test]
 fn git_override_requires_lockfile() {
     VendorPackage::new("git")
-        .file(
-            "Cargo.toml",
-            r#"
-            [package]
-            name = "git"
-            version = "0.5.0"
-            authors = []
-        "#,
-        )
+        .file("Cargo.toml", &basic_manifest("git", "0.5.0"))
         .file("src/lib.rs", "")
         .disable_checksum()
         .build();
 
-    let p = project("bar")
+    let p = project()
         .file(
             "Cargo.toml",
             r#"
@@ -793,9 +643,9 @@ fn git_override_requires_lockfile() {
     "#
     ));
 
-    assert_that(
-        p.cargo("build"),
-        execs().with_status(101).with_stderr(
+    p.cargo("build")
+        .with_status(101)
+        .with_stderr(
             "\
 error: failed to load source for a dependency on `git`
 
@@ -810,6 +660,106 @@ remove the source replacement configuration, generate a lock file, and then
 restore the source replacement configuration to continue the build
 
 ",
-        ),
-    );
+        )
+        .run();
+}
+
+#[test]
+fn workspace_different_locations() {
+    let p = project()
+        .no_manifest()
+        .file(
+            "foo/Cargo.toml",
+            r#"
+                [package]
+                name = 'foo'
+                version = '0.1.0'
+
+                [dependencies]
+                baz = "*"
+            "#,
+        )
+        .file("foo/src/lib.rs", "")
+        .file("foo/vendor/baz/Cargo.toml", &basic_manifest("baz", "0.1.0"))
+        .file("foo/vendor/baz/src/lib.rs", "")
+        .file("foo/vendor/baz/.cargo-checksum.json", "{\"files\":{}}")
+        .file(
+            "bar/Cargo.toml",
+            r#"
+                [package]
+                name = 'bar'
+                version = '0.1.0'
+
+                [dependencies]
+                baz = "*"
+            "#,
+        )
+        .file("bar/src/lib.rs", "")
+        .file(
+            ".cargo/config",
+            r#"
+                [build]
+                target-dir = './target'
+
+                [source.crates-io]
+                replace-with = 'my-awesome-local-registry'
+
+                [source.my-awesome-local-registry]
+                directory = 'foo/vendor'
+            "#,
+        )
+        .build();
+
+    p.cargo("build").cwd("foo").run();
+    p.cargo("build")
+        .cwd("bar")
+        .with_stderr(
+            "\
+[COMPILING] bar [..]
+[FINISHED] [..]
+",
+        )
+        .run();
+}
+
+#[test]
+fn version_missing() {
+    setup();
+
+    VendorPackage::new("foo")
+        .file("src/lib.rs", "pub fn foo() {}")
+        .build();
+
+    VendorPackage::new("bar")
+        .file(
+            "Cargo.toml",
+            r#"
+                [package]
+                name = "bar"
+                version = "0.1.0"
+                authors = []
+
+                [dependencies]
+                foo = "2"
+            "#,
+        )
+        .file("src/main.rs", "fn main() {}")
+        .build();
+
+    cargo_process("install bar")
+        .with_stderr(
+            "\
+[INSTALLING] bar v0.1.0
+error: failed to compile [..]
+
+Caused by:
+  failed to select a version for the requirement `foo = \"^2\"`
+  candidate versions found which didn't match: 0.0.1
+  location searched: directory source `[..] (which is replacing registry `[..]`)
+required by package `bar v0.1.0`
+perhaps a crate was updated and forgotten to be re-vendored?
+",
+        )
+        .with_status(101)
+        .run();
 }
