@@ -1,18 +1,11 @@
-use support::{basic_lib_manifest, is_nightly, execs, project};
-use support::ChannelChanger;
-use support::hamcrest::assert_that;
+use crate::support::{basic_lib_manifest, project};
 
 #[test]
 fn edition_works_for_build_script() {
-    if !is_nightly() {
-        return
-    }
-
     let p = project()
         .file(
             "Cargo.toml",
             r#"
-                cargo-features = ['edition']
                 [package]
                 name = 'foo'
                 version = '0.1.0'
@@ -35,8 +28,5 @@ fn edition_works_for_build_script() {
         .file("a/src/lib.rs", "pub fn foo() {}")
         .build();
 
-    assert_that(
-        p.cargo("build -v").masquerade_as_nightly_cargo(),
-        execs().with_status(0),
-    );
+    p.cargo("build -v").masquerade_as_nightly_cargo().run();
 }
