@@ -1,7 +1,5 @@
+use crate::support::{basic_bin_manifest, basic_lib_manifest, project, Project};
 use cargo::util::paths::dylib_path_envvar;
-use support::{self, ChannelChanger};
-use support::{basic_bin_manifest, basic_lib_manifest, execs, project, Project, path2url};
-use support::hamcrest::{assert_that, existing_file};
 
 #[test]
 fn simple() {
@@ -9,19 +7,16 @@ fn simple() {
         .file("src/main.rs", r#"fn main() { println!("hello"); }"#)
         .build();
 
-    assert_that(
-        p.cargo("run"),
-        execs()
-            .with_stderr(&format!(
-                "\
-[COMPILING] foo v0.0.1 ({dir})
+    p.cargo("run")
+        .with_stderr(
+            "\
+[COMPILING] foo v0.0.1 ([CWD])
 [FINISHED] dev [unoptimized + debuginfo] target(s) in [..]
 [RUNNING] `target/debug/foo[EXE]`",
-                dir = path2url(p.root())
-            ))
-            .with_stdout("hello"),
-    );
-    assert_that(&p.bin("foo"), existing_file());
+        )
+        .with_stdout("hello")
+        .run();
+    assert!(p.bin("foo").is_file());
 }
 
 #[test]
@@ -30,15 +25,9 @@ fn simple_quiet() {
         .file("src/main.rs", r#"fn main() { println!("hello"); }"#)
         .build();
 
-    assert_that(
-        p.cargo("run -q"),
-        execs().with_stdout("hello"),
-    );
+    p.cargo("run -q").with_stdout("hello").run();
 
-    assert_that(
-        p.cargo("run --quiet"),
-        execs().with_stdout("hello"),
-    );
+    p.cargo("run --quiet").with_stdout("hello").run();
 }
 
 #[test]
@@ -47,12 +36,10 @@ fn simple_quiet_and_verbose() {
         .file("src/main.rs", r#"fn main() { println!("hello"); }"#)
         .build();
 
-    assert_that(
-        p.cargo("run").arg("-q").arg("-v"),
-        execs()
-            .with_status(101)
-            .with_stderr("[ERROR] cannot set both --verbose and --quiet"),
-    );
+    p.cargo("run -q -v")
+        .with_status(101)
+        .with_stderr("[ERROR] cannot set both --verbose and --quiet")
+        .run();
 }
 
 #[test]
@@ -68,7 +55,7 @@ fn quiet_and_verbose_config() {
         .file("src/main.rs", r#"fn main() { println!("hello"); }"#)
         .build();
 
-    assert_that(p.cargo("run").arg("-q"), execs());
+    p.cargo("run -q").run();
 }
 
 #[test]
@@ -85,10 +72,7 @@ fn simple_with_args() {
         )
         .build();
 
-    assert_that(
-        p.cargo("run").arg("hello").arg("world"),
-        execs(),
-    );
+    p.cargo("run hello world").run();
 }
 
 #[test]
@@ -99,7 +83,7 @@ fn exit_code() {
 
     let mut output = String::from(
         "\
-[COMPILING] foo v0.0.1 (file[..])
+[COMPILING] foo v0.0.1 ([CWD])
 [FINISHED] dev [unoptimized + debuginfo] target(s) in [..]
 [RUNNING] `target[..]`
 ",
@@ -109,7 +93,7 @@ fn exit_code() {
             "[ERROR] process didn't exit successfully: `target[..]foo[..]` (exit code: 2)",
         );
     }
-    assert_that(p.cargo("run"), execs().with_status(2).with_stderr(output));
+    p.cargo("run").with_status(2).with_stderr(output).run();
 }
 
 #[test]
@@ -120,7 +104,7 @@ fn exit_code_verbose() {
 
     let mut output = String::from(
         "\
-[COMPILING] foo v0.0.1 (file[..])
+[COMPILING] foo v0.0.1 ([CWD])
 [RUNNING] `rustc [..]`
 [FINISHED] dev [unoptimized + debuginfo] target(s) in [..]
 [RUNNING] `target[..]`
@@ -132,25 +116,20 @@ fn exit_code_verbose() {
         );
     }
 
-    assert_that(
-        p.cargo("run").arg("-v"),
-        execs().with_status(2).with_stderr(output),
-    );
+    p.cargo("run -v").with_status(2).with_stderr(output).run();
 }
 
 #[test]
 fn no_main_file() {
-    let p = project()
-        .file("src/lib.rs", "")
-        .build();
+    let p = project().file("src/lib.rs", "").build();
 
-    assert_that(
-        p.cargo("run"),
-        execs().with_status(101).with_stderr(
+    p.cargo("run")
+        .with_status(101)
+        .with_stderr(
             "[ERROR] a bin target must be available \
              for `cargo run`\n",
-        ),
-    );
+        )
+        .run();
 }
 
 #[test]
@@ -161,26 +140,27 @@ fn too_many_bins() {
         .file("src/bin/b.rs", "")
         .build();
 
-    assert_that(
-        p.cargo("run"),
-        // Using [..] here because the order is not stable
-        execs().with_status(101).with_stderr(
-            "[ERROR] `cargo run` requires that a project only \
+    // Using [..] here because the order is not stable
+    p.cargo("run")
+        .with_status(101)
+        .with_stderr(
+            "[ERROR] `cargo run` requires that a package only \
              have one executable; use the `--bin` option \
              to specify which one to run\navailable binaries: [..]\n",
-        ),
-    );
+        )
+        .run();
 
-    assert_that(
-        p.cargo("run").masquerade_as_nightly_cargo(),
-        // Using [..] here because the order is not stable
-        execs().with_status(101).with_stderr(
+    // Using [..] here because the order is not stable
+    p.cargo("run")
+        .masquerade_as_nightly_cargo()
+        .with_status(101)
+        .with_stderr(
             "[ERROR] `cargo run` could not determine which binary to run. \
              Use the `--bin` option to specify a binary, or (on \
              nightly) the `default-run` manifest key.\
              \navailable binaries: [..]\n",
-        ),
-    );
+        )
+        .run();
 }
 
 #[test]
@@ -205,33 +185,28 @@ fn specify_name() {
         )
         .build();
 
-    assert_that(
-        p.cargo("run").arg("--bin").arg("a").arg("-v"),
-        execs()
-            .with_stderr(&format!(
-                "\
-[COMPILING] foo v0.0.1 ({dir})
+    p.cargo("run --bin a -v")
+        .with_stderr(
+            "\
+[COMPILING] foo v0.0.1 ([CWD])
 [RUNNING] `rustc [..] src/lib.rs [..]`
 [RUNNING] `rustc [..] src/bin/a.rs [..]`
 [FINISHED] dev [unoptimized + debuginfo] target(s) in [..]
 [RUNNING] `target/debug/a[EXE]`",
-                dir = path2url(p.root())
-            ))
-            .with_stdout("hello a.rs"),
-    );
+        )
+        .with_stdout("hello a.rs")
+        .run();
 
-    assert_that(
-        p.cargo("run").arg("--bin").arg("b").arg("-v"),
-        execs()
-            .with_stderr(
-                "\
+    p.cargo("run --bin b -v")
+        .with_stderr(
+            "\
 [COMPILING] foo v0.0.1 ([..])
 [RUNNING] `rustc [..] src/bin/b.rs [..]`
 [FINISHED] dev [unoptimized + debuginfo] target(s) in [..]
 [RUNNING] `target/debug/b[EXE]`",
-            )
-            .with_stdout("hello b.rs"),
-    );
+        )
+        .with_stdout("hello b.rs")
+        .run();
 }
 
 #[test]
@@ -254,21 +229,18 @@ fn specify_default_run() {
         .file("src/bin/b.rs", r#"fn main() { println!("hello B"); }"#)
         .build();
 
-    assert_that(
-        p.cargo("run").masquerade_as_nightly_cargo(),
-        execs()
-            .with_stdout("hello A"),
-    );
-    assert_that(
-        p.cargo("run").masquerade_as_nightly_cargo().arg("--bin").arg("a"),
-        execs()
-            .with_stdout("hello A"),
-    );
-    assert_that(
-        p.cargo("run").masquerade_as_nightly_cargo().arg("--bin").arg("b"),
-        execs()
-            .with_stdout("hello B"),
-    );
+    p.cargo("run")
+        .masquerade_as_nightly_cargo()
+        .with_stdout("hello A")
+        .run();
+    p.cargo("run --bin a")
+        .masquerade_as_nightly_cargo()
+        .with_stdout("hello A")
+        .run();
+    p.cargo("run --bin b")
+        .masquerade_as_nightly_cargo()
+        .with_stdout("hello B")
+        .run();
 }
 
 #[test]
@@ -290,12 +262,11 @@ fn bogus_default_run() {
         .file("src/bin/a.rs", r#"fn main() { println!("hello A"); }"#)
         .build();
 
-    assert_that(
-        p.cargo("run").masquerade_as_nightly_cargo(),
-        execs().with_status(101).with_stderr(
-            "error: no bin target named `b`\n\nDid you mean [..]?",
-        ),
-    );
+    p.cargo("run")
+        .masquerade_as_nightly_cargo()
+        .with_status(101)
+        .with_stderr("error: no bin target named `b`\n\nDid you mean [..]?")
+        .run();
 }
 
 #[test]
@@ -314,10 +285,10 @@ fn default_run_unstable() {
         .file("src/bin/a.rs", r#"fn main() { println!("hello A"); }"#)
         .build();
 
-    assert_that(
-        p.cargo("run"),
-        execs().with_status(101).with_stderr(
-r#"error: failed to parse manifest at [..]
+    p.cargo("run")
+        .with_status(101)
+        .with_stderr(
+            r#"error: failed to parse manifest at [..]
 
 Caused by:
   the `default-run` manifest key is unstable
@@ -329,13 +300,14 @@ this Cargo does not support nightly features, but if you
 switch to nightly channel you can add
 `cargo-features = ["default-run"]` to enable this feature
 "#,
-        ),
-    );
+        )
+        .run();
 
-    assert_that(
-        p.cargo("run").masquerade_as_nightly_cargo(),
-        execs().with_status(101).with_stderr(
-r#"error: failed to parse manifest at [..]
+    p.cargo("run")
+        .masquerade_as_nightly_cargo()
+        .with_status(101)
+        .with_stderr(
+            r#"error: failed to parse manifest at [..]
 
 Caused by:
   the `default-run` manifest key is unstable
@@ -345,8 +317,8 @@ Caused by:
 
 consider adding `cargo-features = ["default-run"]` to the manifest
 "#,
-        ),
-    );
+        )
+        .run();
 }
 
 #[test]
@@ -357,18 +329,15 @@ fn run_example() {
         .file("src/bin/a.rs", r#"fn main() { println!("bin"); }"#)
         .build();
 
-    assert_that(
-        p.cargo("run").arg("--example").arg("a"),
-        execs()
-            .with_stderr(&format!(
-                "\
-[COMPILING] foo v0.0.1 ({dir})
+    p.cargo("run --example a")
+        .with_stderr(
+            "\
+[COMPILING] foo v0.0.1 ([CWD])
 [FINISHED] dev [unoptimized + debuginfo] target(s) in [..]
 [RUNNING] `target/debug/examples/a[EXE]`",
-                dir = path2url(p.root())
-            ))
-            .with_stdout("example"),
-    );
+        )
+        .with_stdout("example")
+        .run();
 }
 
 #[test]
@@ -390,12 +359,39 @@ fn run_library_example() {
         .file("examples/bar.rs", "fn foo() {}")
         .build();
 
-    assert_that(
-        p.cargo("run").arg("--example").arg("bar"),
-        execs()
-            .with_status(101)
-            .with_stderr("[ERROR] example target `bar` is a library and cannot be executed"),
-    );
+    p.cargo("run --example bar")
+        .with_status(101)
+        .with_stderr("[ERROR] example target `bar` is a library and cannot be executed")
+        .run();
+}
+
+#[test]
+fn run_bin_example() {
+    let p = project()
+        .file(
+            "Cargo.toml",
+            r#"
+            [package]
+            name = "foo"
+            version = "0.0.1"
+            [[example]]
+            name = "bar"
+            crate_type = ["bin"]
+        "#,
+        )
+        .file("src/lib.rs", "")
+        .file("examples/bar.rs", r#"fn main() { println!("example"); }"#)
+        .build();
+
+    p.cargo("run --example bar")
+        .with_stderr(
+            "\
+[COMPILING] foo v0.0.1 ([CWD])
+[FINISHED] dev [unoptimized + debuginfo] target(s) in [..]
+[RUNNING] `target/debug/examples/bar[EXE]`",
+        )
+        .with_stdout("example")
+        .run();
 }
 
 fn autodiscover_examples_project(rust_edition: &str, autoexamples: Option<bool>) -> Project {
@@ -408,22 +404,20 @@ fn autodiscover_examples_project(rust_edition: &str, autoexamples: Option<bool>)
             "Cargo.toml",
             &format!(
                 r#"
-            cargo-features = ["edition"]
+                    [project]
+                    name = "foo"
+                    version = "0.0.1"
+                    authors = []
+                    edition = "{rust_edition}"
+                    {autoexamples}
 
-            [project]
-            name = "foo"
-            version = "0.0.1"
-            authors = []
-            edition = "{rust_edition}"
-            {autoexamples}
+                    [features]
+                    magic = []
 
-            [features]
-            magic = []
-
-            [[example]]
-            name = "do_magic"
-            required-features = ["magic"]
-        "#,
+                    [[example]]
+                    name = "do_magic"
+                    required-features = ["magic"]
+                "#,
                 rust_edition = rust_edition,
                 autoexamples = autoexamples
             ),
@@ -432,25 +426,18 @@ fn autodiscover_examples_project(rust_edition: &str, autoexamples: Option<bool>)
         .file(
             "examples/do_magic.rs",
             r#"
-            fn main() { println!("magic example"); }
-        "#,
+                fn main() { println!("magic example"); }
+            "#,
         )
         .build()
 }
 
 #[test]
 fn run_example_autodiscover_2015() {
-    if !support::is_nightly() {
-        return;
-    }
-
     let p = autodiscover_examples_project("2015", None);
-    assert_that(
-        p.cargo("run")
-            .arg("--example")
-            .arg("a")
-            .masquerade_as_nightly_cargo(),
-        execs().with_status(101).with_stderr(
+    p.cargo("run --example a")
+        .with_status(101)
+        .with_stderr(
             "warning: \
 An explicit [[example]] section is specified in Cargo.toml which currently
 disables Cargo from automatically inferring other example targets.
@@ -469,74 +456,67 @@ For more information on this warning you can consult
 https://github.com/rust-lang/cargo/issues/5330
 error: no example target named `a`
 ",
-        ),
-    );
+        )
+        .run();
 }
 
 #[test]
 fn run_example_autodiscover_2015_with_autoexamples_enabled() {
-    if !support::is_nightly() {
-        return;
-    }
-
     let p = autodiscover_examples_project("2015", Some(true));
-    assert_that(
-        p.cargo("run")
-            .arg("--example")
-            .arg("a")
-            .masquerade_as_nightly_cargo(),
-        execs()
-            .with_stderr(&format!(
-                "\
-[COMPILING] foo v0.0.1 ({dir})
+    p.cargo("run --example a")
+        .with_stderr(
+            "\
+[COMPILING] foo v0.0.1 ([CWD])
 [FINISHED] dev [unoptimized + debuginfo] target(s) in [..]
 [RUNNING] `target/debug/examples/a[EXE]`",
-                dir = path2url(p.root())
-            ))
-            .with_stdout("example"),
-    );
+        )
+        .with_stdout("example")
+        .run();
 }
 
 #[test]
 fn run_example_autodiscover_2015_with_autoexamples_disabled() {
-    if !support::is_nightly() {
-        return;
-    }
-
     let p = autodiscover_examples_project("2015", Some(false));
-    assert_that(
-        p.cargo("run")
-            .arg("--example")
-            .arg("a")
-            .masquerade_as_nightly_cargo(),
-        execs()
-            .with_status(101)
-            .with_stderr("error: no example target named `a`\n"),
-    );
+    p.cargo("run --example a")
+        .with_status(101)
+        .with_stderr("error: no example target named `a`\n")
+        .run();
 }
 
 #[test]
 fn run_example_autodiscover_2018() {
-    if !support::is_nightly() {
-        return;
-    }
-
     let p = autodiscover_examples_project("2018", None);
-    assert_that(
-        p.cargo("run")
-            .arg("--example")
-            .arg("a")
-            .masquerade_as_nightly_cargo(),
-        execs()
-            .with_stderr(&format!(
-                "\
-[COMPILING] foo v0.0.1 ({dir})
+    p.cargo("run --example a")
+        .with_stderr(
+            "\
+[COMPILING] foo v0.0.1 ([CWD])
 [FINISHED] dev [unoptimized + debuginfo] target(s) in [..]
 [RUNNING] `target/debug/examples/a[EXE]`",
-                dir = path2url(p.root())
-            ))
-            .with_stdout("example"),
-    );
+        )
+        .with_stdout("example")
+        .run();
+}
+
+#[test]
+fn autobins_disables() {
+    let p = project()
+        .file(
+            "Cargo.toml",
+            r#"
+            [project]
+            name = "foo"
+            version = "0.0.1"
+            autobins = false
+            "#,
+        )
+        .file("src/lib.rs", "pub mod bin;")
+        .file("src/bin/mod.rs", "// empty")
+        .build();
+
+    p.cargo("run")
+        .with_status(101)
+        .with_stderr("[ERROR] a bin target must be available for `cargo run`")
+        .run();
 }
 
 #[test]
@@ -547,12 +527,12 @@ fn run_bins() {
         .file("src/bin/a.rs", r#"fn main() { println!("bin"); }"#)
         .build();
 
-    assert_that(
-        p.cargo("run").arg("--bins"),
-        execs().with_status(1).with_stderr_contains(
+    p.cargo("run --bins")
+        .with_status(1)
+        .with_stderr_contains(
             "error: Found argument '--bins' which wasn't expected, or isn't valid in this context",
-        ),
-    );
+        )
+        .run();
 }
 
 #[test]
@@ -569,39 +549,35 @@ fn run_with_filename() {
         .file("examples/a.rs", r#"fn main() { println!("example"); }"#)
         .build();
 
-    assert_that(
-        p.cargo("run").arg("--bin").arg("bin.rs"),
-        execs()
-            .with_status(101)
-            .with_stderr("[ERROR] no bin target named `bin.rs`"),
-    );
+    p.cargo("run --bin bin.rs")
+        .with_status(101)
+        .with_stderr("[ERROR] no bin target named `bin.rs`")
+        .run();
 
-    assert_that(
-        p.cargo("run").arg("--bin").arg("a.rs"),
-        execs().with_status(101).with_stderr(
+    p.cargo("run --bin a.rs")
+        .with_status(101)
+        .with_stderr(
             "\
 [ERROR] no bin target named `a.rs`
 
 Did you mean `a`?",
-        ),
-    );
+        )
+        .run();
 
-    assert_that(
-        p.cargo("run").arg("--example").arg("example.rs"),
-        execs()
-            .with_status(101)
-            .with_stderr("[ERROR] no example target named `example.rs`"),
-    );
+    p.cargo("run --example example.rs")
+        .with_status(101)
+        .with_stderr("[ERROR] no example target named `example.rs`")
+        .run();
 
-    assert_that(
-        p.cargo("run").arg("--example").arg("a.rs"),
-        execs().with_status(101).with_stderr(
+    p.cargo("run --example a.rs")
+        .with_status(101)
+        .with_stderr(
             "\
 [ERROR] no example target named `a.rs`
 
 Did you mean `a`?",
-        ),
-    );
+        )
+        .run();
 }
 
 #[test]
@@ -611,41 +587,37 @@ fn either_name_or_example() {
         .file("examples/b.rs", r#"fn main() { println!("hello b.rs"); }"#)
         .build();
 
-    assert_that(
-        p.cargo("run")
-            .arg("--bin")
-            .arg("a")
-            .arg("--example")
-            .arg("b"),
-        execs().with_status(101).with_stderr(
+    p.cargo("run --bin a --example b")
+        .with_status(101)
+        .with_stderr(
             "[ERROR] `cargo run` can run at most one \
              executable, but multiple were \
              specified",
-        ),
-    );
+        )
+        .run();
 }
 
 #[test]
 fn one_bin_multiple_examples() {
     let p = project()
         .file("src/lib.rs", "")
-        .file("src/bin/main.rs", r#"fn main() { println!("hello main.rs"); }"#)
+        .file(
+            "src/bin/main.rs",
+            r#"fn main() { println!("hello main.rs"); }"#,
+        )
         .file("examples/a.rs", r#"fn main() { println!("hello a.rs"); }"#)
         .file("examples/b.rs", r#"fn main() { println!("hello b.rs"); }"#)
         .build();
 
-    assert_that(
-        p.cargo("run"),
-        execs()
-            .with_stderr(&format!(
-                "\
-[COMPILING] foo v0.0.1 ({dir})
+    p.cargo("run")
+        .with_stderr(
+            "\
+[COMPILING] foo v0.0.1 ([CWD])
 [FINISHED] dev [unoptimized + debuginfo] target(s) in [..]
 [RUNNING] `target/debug/main[EXE]`",
-                dir = path2url(p.root())
-            ))
-            .with_stdout("hello main.rs"),
-    );
+        )
+        .with_stdout("hello main.rs")
+        .run();
 }
 
 #[test]
@@ -694,75 +666,63 @@ fn example_with_release_flag() {
         )
         .build();
 
-    assert_that(
-        p.cargo("run")
-            .arg("-v")
-            .arg("--release")
-            .arg("--example")
-            .arg("a"),
-        execs()
-            .with_stderr(&format!(
-                "\
-[COMPILING] bar v0.5.0 ({url}/bar)
-[RUNNING] `rustc --crate-name bar bar/src/bar.rs --crate-type lib \
+    p.cargo("run -v --release --example a")
+        .with_stderr(
+            "\
+[COMPILING] bar v0.5.0 ([CWD]/bar)
+[RUNNING] `rustc --crate-name bar bar/src/bar.rs --color never --crate-type lib \
         --emit=dep-info,link \
         -C opt-level=3 \
         -C metadata=[..] \
-        --out-dir {dir}/target/release/deps \
-        -L dependency={dir}/target/release/deps`
-[COMPILING] foo v0.0.1 ({url})
-[RUNNING] `rustc --crate-name a examples/a.rs --crate-type bin \
+        --out-dir [CWD]/target/release/deps \
+        -L dependency=[CWD]/target/release/deps`
+[COMPILING] foo v0.0.1 ([CWD])
+[RUNNING] `rustc --crate-name a examples/a.rs --color never --crate-type bin \
         --emit=dep-info,link \
         -C opt-level=3 \
         -C metadata=[..] \
-        --out-dir {dir}/target/release/examples \
-        -L dependency={dir}/target/release/deps \
-         --extern bar={dir}/target/release/deps/libbar-[..].rlib`
+        --out-dir [CWD]/target/release/examples \
+        -L dependency=[CWD]/target/release/deps \
+         --extern bar=[CWD]/target/release/deps/libbar-[..].rlib`
 [FINISHED] release [optimized] target(s) in [..]
 [RUNNING] `target/release/examples/a[EXE]`
 ",
-                dir = p.root().display(),
-                url = path2url(p.root()),
-            ))
-            .with_stdout(
-                "\
+        )
+        .with_stdout(
+            "\
 fast1
 fast2",
-            ),
-    );
+        )
+        .run();
 
-    assert_that(
-        p.cargo("run").arg("-v").arg("--example").arg("a"),
-        execs()
-            .with_stderr(&format!(
-                "\
-[COMPILING] bar v0.5.0 ({url}/bar)
-[RUNNING] `rustc --crate-name bar bar/src/bar.rs --crate-type lib \
+    p.cargo("run -v --example a")
+        .with_stderr(
+            "\
+[COMPILING] bar v0.5.0 ([CWD]/bar)
+[RUNNING] `rustc --crate-name bar bar/src/bar.rs --color never --crate-type lib \
         --emit=dep-info,link \
         -C debuginfo=2 \
         -C metadata=[..] \
-        --out-dir {dir}/target/debug/deps \
-        -L dependency={dir}/target/debug/deps`
-[COMPILING] foo v0.0.1 ({url})
-[RUNNING] `rustc --crate-name a examples/a.rs --crate-type bin \
+        --out-dir [CWD]/target/debug/deps \
+        -L dependency=[CWD]/target/debug/deps`
+[COMPILING] foo v0.0.1 ([CWD])
+[RUNNING] `rustc --crate-name a examples/a.rs --color never --crate-type bin \
         --emit=dep-info,link \
         -C debuginfo=2 \
         -C metadata=[..] \
-        --out-dir {dir}/target/debug/examples \
-        -L dependency={dir}/target/debug/deps \
-         --extern bar={dir}/target/debug/deps/libbar-[..].rlib`
+        --out-dir [CWD]/target/debug/examples \
+        -L dependency=[CWD]/target/debug/deps \
+         --extern bar=[CWD]/target/debug/deps/libbar-[..].rlib`
 [FINISHED] dev [unoptimized + debuginfo] target(s) in [..]
 [RUNNING] `target/debug/examples/a[EXE]`
 ",
-                dir = p.root().display(),
-                url = path2url(p.root()),
-            ))
-            .with_stdout(
-                "\
+        )
+        .with_stdout(
+            "\
 slow1
 slow2",
-            ),
-    );
+        )
+        .run();
 }
 
 #[test]
@@ -780,7 +740,10 @@ fn run_dylib_dep() {
             path = "bar"
         "#,
         )
-        .file("src/main.rs", r#"extern crate bar; fn main() { bar::bar(); }"#)
+        .file(
+            "src/main.rs",
+            r#"extern crate bar; fn main() { bar::bar(); }"#,
+        )
         .file(
             "bar/Cargo.toml",
             r#"
@@ -797,10 +760,7 @@ fn run_dylib_dep() {
         .file("bar/src/lib.rs", "pub fn bar() {}")
         .build();
 
-    assert_that(
-        p.cargo("run").arg("hello").arg("world"),
-        execs(),
-    );
+    p.cargo("run hello world").run();
 }
 
 #[test]
@@ -814,18 +774,16 @@ fn release_works() {
         )
         .build();
 
-    assert_that(
-        p.cargo("run").arg("--release"),
-        execs().with_stderr(&format!(
+    p.cargo("run --release")
+        .with_stderr(
             "\
-[COMPILING] foo v0.0.1 ({dir})
+[COMPILING] foo v0.0.1 ([CWD])
 [FINISHED] release [optimized] target(s) in [..]
 [RUNNING] `target/release/foo[EXE]`
 ",
-            dir = path2url(p.root()),
-        )),
-    );
-    assert_that(&p.release_bin("foo"), existing_file());
+        )
+        .run();
+    assert!(p.release_bin("foo").is_file());
 }
 
 #[test]
@@ -846,7 +804,7 @@ fn run_bin_different_name() {
         .file("src/bar.rs", "fn main() {}")
         .build();
 
-    assert_that(p.cargo("run"), execs());
+    p.cargo("run").run();
 }
 
 #[test]
@@ -866,10 +824,7 @@ fn dashes_are_forwarded() {
         )
         .build();
 
-    assert_that(
-        p.cargo("run -- -- a -- b"),
-        execs(),
-    );
+    p.cargo("run -- -- a -- b").run();
 }
 
 #[test]
@@ -879,18 +834,17 @@ fn run_from_executable_folder() {
         .build();
 
     let cwd = p.root().join("target").join("debug");
-    p.cargo("build").exec_with_output().unwrap();
+    p.cargo("build").run();
 
-    assert_that(
-        p.cargo("run").cwd(cwd),
-        execs()
-            .with_stderr(
-                "\
-                 [FINISHED] dev [unoptimized + debuginfo] target(s) in [..]\n\
-                 [RUNNING] `./foo[EXE]`",
-            )
-            .with_stdout("hello"),
-    );
+    p.cargo("run")
+        .cwd(cwd)
+        .with_stderr(
+            "\
+             [FINISHED] dev [unoptimized + debuginfo] target(s) in [..]\n\
+             [RUNNING] `./foo[EXE]`",
+        )
+        .with_stdout("hello")
+        .run();
 }
 
 #[test]
@@ -905,16 +859,18 @@ fn run_with_library_paths() {
     let mut dir2 = p.target_debug_dir();
     dir2.push("dir=containing=equal=signs");
 
-    let p = p.file(
-        "Cargo.toml",
-        r#"
+    let p = p
+        .file(
+            "Cargo.toml",
+            r#"
             [project]
             name = "foo"
             version = "0.0.1"
             authors = []
             build = "build.rs"
         "#,
-    ).file(
+        )
+        .file(
             "build.rs",
             &format!(
                 r##"
@@ -945,7 +901,7 @@ fn run_with_library_paths() {
         )
         .build();
 
-    assert_that(p.cargo("run"), execs());
+    p.cargo("run").run();
 }
 
 #[test]
@@ -961,16 +917,18 @@ fn library_paths_sorted_alphabetically() {
     let mut dir3 = p.target_debug_dir();
     dir3.push("aaaaaaa");
 
-    let p = p.file(
-        "Cargo.toml",
-        r#"
+    let p = p
+        .file(
+            "Cargo.toml",
+            r#"
             [project]
             name = "foo"
             version = "0.0.1"
             authors = []
             build = "build.rs"
         "#,
-    ).file(
+        )
+        .file(
             "build.rs",
             &format!(
                 r##"
@@ -1003,7 +961,7 @@ fn library_paths_sorted_alphabetically() {
         )
         .build();
 
-    assert_that(p.cargo("run"), execs());
+    p.cargo("run").run();
 }
 
 #[test]
@@ -1012,10 +970,11 @@ fn fail_no_extra_verbose() {
         .file("src/main.rs", "fn main() { std::process::exit(1); }")
         .build();
 
-    assert_that(
-        p.cargo("run").arg("-q"),
-        execs().with_status(1).with_stdout("").with_stderr(""),
-    );
+    p.cargo("run -q")
+        .with_status(1)
+        .with_stdout("")
+        .with_stderr("")
+        .run();
 }
 
 #[test]
@@ -1053,33 +1012,32 @@ fn run_multiple_packages() {
 
     let cargo = || {
         let mut process_builder = p.cargo("run");
-        process_builder.cwd(p.root().join("foo"));
+        process_builder.cwd("foo");
         process_builder
     };
 
-    assert_that(
-        cargo().arg("-p").arg("d1"),
-        execs().with_stdout("d1"),
-    );
+    cargo().arg("-p").arg("d1").with_stdout("d1").run();
 
-    assert_that(
-        cargo().arg("-p").arg("d2").arg("--bin").arg("d2"),
-        execs().with_stdout("d2"),
-    );
+    cargo()
+        .arg("-p")
+        .arg("d2")
+        .arg("--bin")
+        .arg("d2")
+        .with_stdout("d2")
+        .run();
 
-    assert_that(cargo(), execs().with_stdout("foo"));
+    cargo().with_stdout("foo").run();
 
-    assert_that(cargo().arg("-p").arg("d1").arg("-p").arg("d2"),
-                execs()
+    cargo().arg("-p").arg("d1").arg("-p").arg("d2")
                     .with_status(1)
-                    .with_stderr_contains("error: The argument '--package <SPEC>' was provided more than once, but cannot be used multiple times"));
+                    .with_stderr_contains("error: The argument '--package <SPEC>' was provided more than once, but cannot be used multiple times").run();
 
-    assert_that(
-        cargo().arg("-p").arg("d3"),
-        execs()
-            .with_status(101)
-            .with_stderr_contains("[ERROR] package `d3` is not a member of the workspace"),
-    );
+    cargo()
+        .arg("-p")
+        .arg("d3")
+        .with_status(101)
+        .with_stderr_contains("[ERROR] package `d3` is not a member of the workspace")
+        .run();
 }
 
 #[test]
@@ -1096,5 +1054,146 @@ fn explicit_bin_with_args() {
         )
         .build();
 
-    assert_that(p.cargo("run --bin foo hello world"), execs());
+    p.cargo("run --bin foo hello world").run();
+}
+
+#[test]
+fn run_workspace() {
+    let p = project()
+        .file(
+            "Cargo.toml",
+            r#"
+            [workspace]
+            members = ["a", "b"]
+        "#,
+        )
+        .file("a/Cargo.toml", &basic_bin_manifest("a"))
+        .file("a/src/main.rs", r#"fn main() {println!("run-a");}"#)
+        .file("b/Cargo.toml", &basic_bin_manifest("b"))
+        .file("b/src/main.rs", r#"fn main() {println!("run-b");}"#)
+        .build();
+
+    p.cargo("run")
+        .with_status(101)
+        .with_stderr(
+            "\
+[ERROR] `cargo run` requires that a package only have one executable[..]
+available binaries: a, b",
+        )
+        .run();
+    p.cargo("run --bin a").with_stdout("run-a").run();
+}
+
+#[test]
+fn default_run_workspace() {
+    let p = project()
+        .file(
+            "Cargo.toml",
+            r#"
+            [workspace]
+            members = ["a", "b"]
+        "#,
+        )
+        .file(
+            "a/Cargo.toml",
+            r#"
+            cargo-features = ["default-run"]
+
+            [project]
+            name = "a"
+            version = "0.0.1"
+            default-run = "a"
+        "#,
+        )
+        .file("a/src/main.rs", r#"fn main() {println!("run-a");}"#)
+        .file("b/Cargo.toml", &basic_bin_manifest("b"))
+        .file("b/src/main.rs", r#"fn main() {println!("run-b");}"#)
+        .build();
+
+    p.cargo("run").masquerade_as_nightly_cargo().with_stdout("run-a").run();
+}
+
+#[test]
+#[cfg(target_os = "macos")]
+fn run_link_system_path_macos() {
+    use crate::support::paths::{self, CargoPathExt};
+    use std::fs;
+    // Check that the default system library path is honored.
+    // First, build a shared library that will be accessed from
+    // DYLD_FALLBACK_LIBRARY_PATH.
+    let p = project()
+        .file(
+            "Cargo.toml",
+            r#"
+            [project]
+            name = "foo"
+            version = "0.0.1"
+            [lib]
+            crate-type = ["cdylib"]
+            "#,
+        )
+        .file(
+            "src/lib.rs",
+            "#[no_mangle] pub extern fn something_shared() {}",
+        )
+        .build();
+    p.cargo("build").run();
+
+    // This is convoluted. Since this test can't modify things in /usr,
+    // this needs to dance around to check that things work.
+    //
+    // The default DYLD_FALLBACK_LIBRARY_PATH is:
+    //      $(HOME)/lib:/usr/local/lib:/lib:/usr/lib
+    //
+    // This will make use of ~/lib in the path, but the default cc link
+    // path is /usr/lib:/usr/local/lib. So first need to build in one
+    // location, and then move it to ~/lib.
+    //
+    // 1. Build with rustc-link-search pointing to libfoo so the initial
+    //    binary can be linked.
+    // 2. Move the library to ~/lib
+    // 3. Run `cargo run` to make sure it can still find the library in
+    //    ~/lib.
+    //
+    // This should be equivalent to having the library in /usr/local/lib.
+    let p2 = project()
+        .at("bar")
+        .file("Cargo.toml", &basic_bin_manifest("bar"))
+        .file(
+            "src/main.rs",
+            r#"
+            extern {
+                fn something_shared();
+            }
+            fn main() {
+                unsafe { something_shared(); }
+            }
+            "#,
+        )
+        .file(
+            "build.rs",
+            &format!(
+                r#"
+            fn main() {{
+                println!("cargo:rustc-link-lib=foo");
+                println!("cargo:rustc-link-search={}");
+            }}
+            "#,
+                p.target_debug_dir().display()
+            ),
+        )
+        .build();
+    p2.cargo("build").run();
+    p2.cargo("test").run();
+
+    let libdir = paths::home().join("lib");
+    fs::create_dir(&libdir).unwrap();
+    fs::rename(
+        p.target_debug_dir().join("libfoo.dylib"),
+        libdir.join("libfoo.dylib"),
+    )
+    .unwrap();
+    p.root().rm_rf();
+    p2.cargo("run").run();
+    p2.cargo("test").run();
 }
