@@ -1,6 +1,3 @@
-#![feature(tool_lints)]
-
-
 #![deny(clippy::borrowed_box)]
 #![allow(clippy::blacklisted_name)]
 #![allow(unused_variables)]
@@ -15,7 +12,7 @@ pub fn test2() {
 }
 
 struct Test3<'a> {
-    foo: &'a Box<bool>
+    foo: &'a Box<bool>,
 }
 
 trait Test4 {
@@ -39,7 +36,7 @@ pub fn test6() {
 }
 
 struct Test7<'a> {
-    foo: &'a Box<Any>
+    foo: &'a Box<Any>,
 }
 
 trait Test8 {
@@ -61,7 +58,7 @@ pub fn test10() {
 }
 
 struct Test11<'a> {
-    foo: &'a Box<Any + Send>
+    foo: &'a Box<Any + Send>,
 }
 
 trait Test12 {
@@ -74,7 +71,7 @@ impl<'a> Test12 for Test11<'a> {
     }
 }
 
-fn main(){
+fn main() {
     test1(&mut Box::new(false));
     test2();
     test5(&mut (Box::new(false) as Box<Any>));
