@@ -29,7 +29,7 @@ impl Unrelated {
     clippy::linkedlist,
     clippy::shadow_unrelated,
     clippy::unnecessary_mut_passed,
-    clippy::cyclomatic_complexity,
+    clippy::cognitive_complexity,
     clippy::similar_names
 )]
 #[allow(clippy::many_single_char_names, unused_variables, clippy::into_iter_on_array)]
@@ -355,114 +355,6 @@ fn partition<T: PartialOrd + Send>(v: &mut [T]) -> usize {
     i
 }
 
-const LOOP_OFFSET: usize = 5000;
-
-#[warn(clippy::needless_range_loop)]
-pub fn manual_copy(src: &[i32], dst: &mut [i32], dst2: &mut [i32]) {
-    // plain manual memcpy
-    for i in 0..src.len() {
-        dst[i] = src[i];
-    }
-
-    // dst offset memcpy
-    for i in 0..src.len() {
-        dst[i + 10] = src[i];
-    }
-
-    // src offset memcpy
-    for i in 0..src.len() {
-        dst[i] = src[i + 10];
-    }
-
-    // src offset memcpy
-    for i in 11..src.len() {
-        dst[i] = src[i - 10];
-    }
-
-    // overwrite entire dst
-    for i in 0..dst.len() {
-        dst[i] = src[i];
-    }
-
-    // manual copy with branch - can't easily convert to memcpy!
-    for i in 0..src.len() {
-        dst[i] = src[i];
-        if dst[i] > 5 {
-            break;
-        }
-    }
-
-    // multiple copies - suggest two memcpy statements
-    for i in 10..256 {
-        dst[i] = src[i - 5];
-        dst2[i + 500] = src[i]
-    }
-
-    // this is a reversal - the copy lint shouldn't be triggered
-    for i in 10..LOOP_OFFSET {
-        dst[i + LOOP_OFFSET] = src[LOOP_OFFSET - i];
-    }
-
-    let some_var = 5;
-    // Offset in variable
-    for i in 10..LOOP_OFFSET {
-        dst[i + LOOP_OFFSET] = src[i - some_var];
-    }
-
-    // Non continuous copy - don't trigger lint
-    for i in 0..10 {
-        dst[i + i] = src[i];
-    }
-
-    let src_vec = vec![1, 2, 3, 4, 5];
-    let mut dst_vec = vec![0, 0, 0, 0, 0];
-
-    // make sure vectors are supported
-    for i in 0..src_vec.len() {
-        dst_vec[i] = src_vec[i];
-    }
-
-    // lint should not trigger when either
-    // source or destination type is not
-    // slice-like, like DummyStruct
-    struct DummyStruct(i32);
-
-    impl ::std::ops::Index<usize> for DummyStruct {
-        type Output = i32;
-
-        fn index(&self, _: usize) -> &i32 {
-            &self.0
-        }
-    }
-
-    let src = DummyStruct(5);
-    let mut dst_vec = vec![0; 10];
-
-    for i in 0..10 {
-        dst_vec[i] = src[i];
-    }
-
-    // Simplify suggestion (issue #3004)
-    let src = [0, 1, 2, 3, 4];
-    let mut dst = [0, 0, 0, 0, 0, 0];
-    let from = 1;
-
-    for i in from..from + src.len() {
-        dst[i] = src[i - from];
-    }
-
-    for i in from..from + 3 {
-        dst[i] = src[i - from];
-    }
-}
-
-#[warn(clippy::needless_range_loop)]
-pub fn manual_clone(src: &[String], dst: &mut [String]) {
-    for i in 0..src.len() {
-        dst[i] = src[i].clone();
-    }
-}
-
 #[warn(clippy::needless_range_loop)]
 pub fn manual_copy_same_destination(dst: &mut [i32], d: usize, s: usize) {
     // Same source and destination - don't trigger lint
