@@ -1,19 +1,16 @@
-#![feature(tool_lints)]
-
-
 #![warn(clippy::let_unit_value)]
 #![allow(unused_variables)]
 
 macro_rules! let_and_return {
     ($n:expr) => {{
         let ret = $n;
-    }}
+    }};
 }
 
 fn main() {
     let _x = println!("x");
-    let _y = 1;   // this is fine
-    let _z = ((), 1);  // this as well
+    let _y = 1; // this is fine
+    let _z = ((), 1); // this as well
     if true {
         let _a = ();
     }
