@@ -1,9 +1,8 @@
-#![feature(tool_lints)]
-
 #![warn(clippy::useless_attribute)]
 
 #[allow(dead_code)]
 #[cfg_attr(feature = "cargo-clippy", allow(dead_code))]
+#[rustfmt::skip]
 #[cfg_attr(feature = "cargo-clippy",
            allow(dead_code))]
 #[allow(unused_imports)]
@@ -16,7 +15,10 @@ extern crate clippy_lints;
 use std::collections;
 
 // don't lint on deprecated for `use` items
-mod foo { #[deprecated] pub struct Bar; }
+mod foo {
+    #[deprecated]
+    pub struct Bar;
+}
 #[allow(deprecated)]
 pub use foo::Bar;
 
