@@ -1,9 +1,8 @@
 use rustc::mir;
+use rustc::mir::interpret::{EvalResult, PointerArithmetic};
 use rustc::ty::layout::{self, LayoutOf, Size};
 use rustc::ty;
 
-use rustc::mir::interpret::{EvalResult, PointerArithmetic};
-
 use crate::{
     PlaceTy, OpTy, ImmTy, Immediate, Scalar, ScalarMaybeUndef, Borrow,
     OperatorEvalContextExt
@@ -28,7 +27,7 @@ pub trait EvalContextExt<'a, 'mir, 'tcx: 'a+'mir>: crate::MiriEvalContextExt<'a,
         // (as opposed to through a place), we have to remember to erase any tag
         // that might still hang around!
 
-        let intrinsic_name = &this.tcx.item_name(instance.def_id()).as_str()[..];
+        let intrinsic_name = this.tcx.item_name(instance.def_id()).as_str().get();
         match intrinsic_name {
             "arith_offset" => {
                 let offset = this.read_scalar(args[1])?.to_isize(this)?;
@@ -50,22 +49,30 @@ pub trait EvalContextExt<'a, 'mir, 'tcx: 'a+'mir>: crate::MiriEvalContextExt<'a,
 
             "atomic_load" |
             "atomic_load_relaxed" |
-            "atomic_load_acq" |
-            "volatile_load" => {
+            "atomic_load_acq" => {
                 let ptr = this.deref_operand(args[0])?;
                 let val = this.read_scalar(ptr.into())?; // make sure it fits into a scalar; otherwise it cannot be atomic
                 this.write_scalar(val, dest)?;
             }
 
+            "volatile_load" => {
+                let ptr = this.deref_operand(args[0])?;
+                this.copy_op(ptr.into(), dest)?;
+            }
+
             "atomic_store" |
             "atomic_store_relaxed" |
-            "atomic_store_rel" |
-            "volatile_store" => {
+            "atomic_store_rel" => {
                 let ptr = this.deref_operand(args[0])?;
                 let val = this.read_scalar(args[1])?; // make sure it fits into a scalar; otherwise it cannot be atomic
                 this.write_scalar(val, ptr.into())?;
             }
 
+            "volatile_store" => {
+                let ptr = this.deref_operand(args[0])?;
+                this.copy_op(args[1], ptr.into())?;
+            }
+
             "atomic_fence_acq" => {
                 // we are inherently singlethreaded and singlecored, this is a nop
             }
@@ -260,7 +267,7 @@ pub trait EvalContextExt<'a, 'mir, 'tcx: 'a+'mir>: crate::MiriEvalContextExt<'a,
                 // However, this only affects direct calls of the intrinsic; calls to the stable
                 // functions wrapping them do get their validation.
                 // FIXME: should we check that the destination pointer is aligned even for ZSTs?
-                if !dest.layout.is_zst() { // nothing to do for ZST
+                if !dest.layout.is_zst() {
                     match dest.layout.abi {
                         layout::Abi::Scalar(ref s) => {
                             let x = Scalar::from_int(0, s.value.size(this));
@@ -435,7 +442,7 @@ pub trait EvalContextExt<'a, 'mir, 'tcx: 'a+'mir>: crate::MiriEvalContextExt<'a,
                 // However, this only affects direct calls of the intrinsic; calls to the stable
                 // functions wrapping them do get their validation.
                 // FIXME: should we check alignment for ZSTs?
-                if !dest.layout.is_zst() { // nothing to do for ZST
+                if !dest.layout.is_zst() {
                     match dest.layout.abi {
                         layout::Abi::Scalar(..) => {
                             let x = ScalarMaybeUndef::Undef;
