@@ -1,5 +1,5 @@
 // This should fail even without validation
-// compile-flags: -Zmir-emit-validate=0
+// compile-flags: -Zmiri-disable-validation
 
 #![allow(dead_code, unused_variables)]
 
@@ -15,6 +15,5 @@ fn main() {
         y: 99,
     };
     let p = unsafe { &foo.x };
-    let i = *p; //~ ERROR constant evaluation error
-    //~^ NOTE tried to access memory with alignment 1, but alignment 4 is required
+    let i = *p; //~ ERROR tried to access memory with alignment 1, but alignment 4 is required
 }
