@@ -1,13 +1,3 @@
-// Copyright 2014 The Rust Project Developers. See the COPYRIGHT
-// file at the top-level directory of this distribution and at
-// http://rust-lang.org/COPYRIGHT.
-//
-// Licensed under the Apache License, Version 2.0 <LICENSE-APACHE or
-// http://www.apache.org/licenses/LICENSE-2.0> or the MIT license
-// <LICENSE-MIT or http://opensource.org/licenses/MIT>, at your
-// option. This file may not be copied, modified, or distributed
-// except according to those terms.
-
 // This is a regression test for the ICE from issue #10846.
 //
 // The original issue causing the ICE: the LUB-computations during
@@ -17,14 +7,11 @@
 //
 // However, those encounters were occurring within the lexical scope
 // of the binding for the late-bound lifetime; that is, the late-bound
-// lifetimes were perfectly valid.  The core problem was that the type
+// lifetimes were perfectly valid. The core problem was that the type
 // folding code was over-zealously passing back all lifetimes when
 // doing region-folding, when really all clients of the region-folding
-// case only want to see FREE lifetime variables, not bound ones.
+// case only want to see *free* lifetime variables, not bound ones.
 
-// pretty-expanded FIXME #23616
-
-#![allow(unused_features)]
 #![feature(box_syntax)]
 
 pub fn main() {
