@@ -1,14 +1,3 @@
-// Copyright 2015 The Rust Project Developers. See the COPYRIGHT
-// file at the top-level directory of this distribution and at
-// http://rust-lang.org/COPYRIGHT.
-//
-// Licensed under the Apache License, Version 2.0 <LICENSE-APACHE or
-// http://www.apache.org/licenses/LICENSE-2.0> or the MIT license
-// <LICENSE-MIT or http://opensource.org/licenses/MIT>, at your
-// option. This file may not be copied, modified, or distributed
-// except according to those terms.
-
-use regex::Regex;
 use std::cell::Cell;
 use std::default::Default;
 use std::fs::File;
@@ -16,10 +5,12 @@ use std::io::{Error, ErrorKind, Read};
 use std::path::{Path, PathBuf};
 use std::{env, fs};
 
-use config::config_type::ConfigType;
-pub use config::file_lines::{FileLines, FileName, Range};
-pub use config::lists::*;
-pub use config::options::*;
+use regex::Regex;
+
+use crate::config::config_type::ConfigType;
+pub use crate::config::file_lines::{FileLines, FileName, Range};
+pub use crate::config::lists::*;
+pub use crate::config::options::*;
 
 #[macro_use]
 pub mod config_type;
@@ -30,10 +21,10 @@ pub mod file_lines;
 pub mod license;
 pub mod lists;
 
-/// This macro defines configuration options used in rustfmt. Each option
-/// is defined as follows:
-///
-/// `name: value type, default value, is stable, description;`
+// This macro defines configuration options used in rustfmt. Each option
+// is defined as follows:
+//
+// `name: value type, default value, is stable, description;`
 create_config! {
     // Fundamental stuff
     max_width: usize, 100, true, "Maximum width of each line";
@@ -64,7 +55,7 @@ create_config! {
     struct_lit_single_line: bool, true, false,
         "Put small struct literals on a single line";
     fn_single_line: bool, false, false, "Put single-expression functions on a single line";
-    where_single_line: bool, false, false, "Force where clauses to be on a single line";
+    where_single_line: bool, false, false, "Force where-clauses to be on a single line";
 
     // Imports
     imports_indent: IndentStyle, IndentStyle::Block, false, "Indent of imports";
@@ -113,6 +104,10 @@ create_config! {
     blank_lines_lower_bound: usize, 0, false,
         "Minimum number of blank lines which must be put between items";
     edition: Edition, Edition::Edition2015, true, "The edition of the parser (RFC 2052)";
+    version: Version, Version::One, false, "Version of formatting rules";
+    inline_attribute_width: usize, 0, false,
+        "Write an item and its attribute on the same line \
+        if their combined width is below a threshold";
 
     // Options that can change the source code beyond whitespace/blocks (somewhat linty things)
     merge_derives: bool, true, true, "Merge multiple `#[derive(...)]` into a single one";
@@ -155,7 +150,143 @@ create_config! {
     make_backup: bool, false, false, "Backup changed files";
 }
 
-/// Load a config by checking the client-supplied options and if appropriate, the
+impl PartialConfig {
+    pub fn to_toml(&self) -> Result<String, String> {
+        // Non-user-facing options can't be specified in TOML
+        let mut cloned = self.clone();
+        cloned.file_lines = None;
+        cloned.verbose = None;
+        cloned.width_heuristics = None;
+
+        ::toml::to_string(&cloned).map_err(|e| format!("Could not output config: {}", e))
+    }
+}
+
+impl Config {
+    pub(crate) fn version_meets_requirement(&self) -> bool {
+        if self.was_set().required_version() {
+            let version = env!("CARGO_PKG_VERSION");
+            let required_version = self.required_version();
+            if version != required_version {
+                println!(
+                    "Error: rustfmt version ({}) doesn't match the required version ({})",
+                    version, required_version,
+                );
+                return false;
+            }
+        }
+
+        true
+    }
+
+    /// Constructs a `Config` from the toml file specified at `file_path`.
+    ///
+    /// This method only looks at the provided path, for a method that
+    /// searches parents for a `rustfmt.toml` see `from_resolved_toml_path`.
+    ///
+    /// Returns a `Config` if the config could be read and parsed from
+    /// the file, otherwise errors.
+    pub(super) fn from_toml_path(file_path: &Path) -> Result<Config, Error> {
+        let mut file = File::open(&file_path)?;
+        let mut toml = String::new();
+        file.read_to_string(&mut toml)?;
+        Config::from_toml(&toml, file_path.parent().unwrap())
+            .map_err(|err| Error::new(ErrorKind::InvalidData, err))
+    }
+
+    /// Resolves the config for input in `dir`.
+    ///
+    /// Searches for `rustfmt.toml` beginning with `dir`, and
+    /// recursively checking parents of `dir` if no config file is found.
+    /// If no config file exists in `dir` or in any parent, a
+    /// default `Config` will be returned (and the returned path will be empty).
+    ///
+    /// Returns the `Config` to use, and the path of the project file if there was
+    /// one.
+    pub(super) fn from_resolved_toml_path(dir: &Path) -> Result<(Config, Option<PathBuf>), Error> {
+        /// Try to find a project file in the given directory and its parents.
+        /// Returns the path of a the nearest project file if one exists,
+        /// or `None` if no project file was found.
+        fn resolve_project_file(dir: &Path) -> Result<Option<PathBuf>, Error> {
+            let mut current = if dir.is_relative() {
+                env::current_dir()?.join(dir)
+            } else {
+                dir.to_path_buf()
+            };
+
+            current = fs::canonicalize(current)?;
+
+            loop {
+                match get_toml_path(&current) {
+                    Ok(Some(path)) => return Ok(Some(path)),
+                    Err(e) => return Err(e),
+                    _ => (),
+                }
+
+                // If the current directory has no parent, we're done searching.
+                if !current.pop() {
+                    break;
+                }
+            }
+
+            // If nothing was found, check in the home directory.
+            if let Some(home_dir) = dirs::home_dir() {
+                if let Some(path) = get_toml_path(&home_dir)? {
+                    return Ok(Some(path));
+                }
+            }
+
+            // If none was found ther either, check in the user's configuration directory.
+            if let Some(mut config_dir) = dirs::config_dir() {
+                config_dir.push("rustfmt");
+                if let Some(path) = get_toml_path(&config_dir)? {
+                    return Ok(Some(path));
+                }
+            }
+
+            return Ok(None);
+        }
+
+        match resolve_project_file(dir)? {
+            None => Ok((Config::default(), None)),
+            Some(path) => Config::from_toml_path(&path).map(|config| (config, Some(path))),
+        }
+    }
+
+    pub(crate) fn from_toml(toml: &str, dir: &Path) -> Result<Config, String> {
+        let parsed: ::toml::Value = toml
+            .parse()
+            .map_err(|e| format!("Could not parse TOML: {}", e))?;
+        let mut err: String = String::new();
+        {
+            let table = parsed
+                .as_table()
+                .ok_or(String::from("Parsed config was not table"))?;
+            for key in table.keys() {
+                if !Config::is_valid_name(key) {
+                    let msg = &format!("Warning: Unknown configuration option `{}`\n", key);
+                    err.push_str(msg)
+                }
+            }
+        }
+        match parsed.try_into() {
+            Ok(parsed_config) => {
+                if !err.is_empty() {
+                    eprint!("{}", err);
+                }
+                Ok(Config::default().fill_from_parsed_config(parsed_config, dir))
+            }
+            Err(e) => {
+                err.push_str("Error: Decoding config file failed:\n");
+                err.push_str(format!("{}\n", e).as_str());
+                err.push_str("Please check your config file.");
+                Err(err)
+            }
+        }
+    }
+}
+
+/// Loads a config by checking the client-supplied options and if appropriate, the
 /// file system (including searching the file system for overrides).
 pub fn load_config<O: CliOptions>(
     file_path: Option<&Path>,
@@ -206,7 +337,7 @@ fn get_toml_path(dir: &Path) -> Result<Option<PathBuf>, Error> {
     Ok(None)
 }
 
-fn config_path(options: &CliOptions) -> Result<Option<PathBuf>, Error> {
+fn config_path(options: &dyn CliOptions) -> Result<Option<PathBuf>, Error> {
     let config_path_not_found = |path: &str| -> Result<Option<PathBuf>, Error> {
         Err(Error::new(
             ErrorKind::NotFound,
@@ -331,7 +462,7 @@ mod test {
         assert_eq!(s.contains("(unstable)"), true);
     }
 
-    // FIXME(#2183) these tests cannot be run in parallel because they use env vars
+    // FIXME(#2183): these tests cannot be run in parallel because they use env vars.
     // #[test]
     // fn test_as_not_nightly_channel() {
     //     let mut config = Config::default();
